// API Endpoints - used to interact with the backend
export const API_ENDPOINTS = {
    ANALYSIS: {
        CORRELATE: '/correlate',
        DISTANCE: '/distance',
        FIND_PEAKS: '/find_peaks',
        WINDOWED_AVERAGE: '/windowed_average',
    },
    CHECK_CACHE: '/check_cache',
    CLEAR_CACHE: '/clear_cache',
    IO: {
        SAVE_SCENE: '/save_scene',
        LOAD_SCENE: '/load_scene',
    },
    DATA: {
        CONVERT_TIMEPOINTS: '/convert_timepoints',
        GET_CLICK_COORDS: '/get_click_coords',
        GET_COORD_LABELS: '/get_coord_labels',
        GET_CROSSHAIR_COORDS: '/get_crosshair_coords',
        GET_DIRECTION_LABEL_COORDS: '/get_direction_label_coords',
        GET_DISTANCE_DATA: '/get_distance_data',
        GET_FMRI_DATA: '/get_fmri_data',
        GET_LAST_TIMECOURSE: '/get_last_timecourse',
        GET_MONTAGE_DATA: '/get_montage_data',
        GET_TASK_CONDITIONS: '/get_task_conditions',
        GET_TIMECOURSE_LABELS: '/get_timecourse_labels',
        GET_TIMECOURSE_LABELS_PREPROCESSED: '/get_timecourse_labels_preprocessed',
        GET_TIMECOURSE_DATA: '/get_timecourse_data',
        GET_TIMECOURSE_SOURCE: '/get_timecourse_source',
        GET_TIMEPOINT: '/get_timepoint',
        GET_TIMEPOINTS: '/get_timepoints',
        GET_VERTEX_COORDS: '/get_vertex_coords',
        GET_VIEWER_METADATA: '/get_viewer_metadata',
        GET_VOXEL_COORDS: '/get_voxel_coords',
        GET_WORLD_COORDS: '/get_world_coords',
        POP_FMRI_TIMECOURSE: '/pop_fmri_timecourse',
        REMOVE_FMRI_TIMECOURSES: '/remove_fmri_timecourses',
        UPDATE_FMRI_TIMECOURSE: '/update_fmri_timecourse',
        UPDATE_LOCATION: '/update_location',
        UPDATE_MONTAGE_SLICE_DIR: '/update_montage_slice_dir',
        UPDATE_MONTAGE_SLICE_IDX: '/update_montage_slice_idx',
        UPDATE_TIMEPOINT: '/update_timepoint',
        UPDATE_TR: '/update_tr',
    },
    PLOT_OPTIONS: {
        ADD_ANNOTATION_MARKER: '/add_annotation_marker',
        CHANGE_TASK_CONVOLUTION: '/change_task_convolution',
        CHECK_FMRI_PREPROCESSED: '/check_fmri_preprocessed',
        CHECK_TS_PREPROCESSED: '/check_ts_preprocessed',
        CLEAR_ANNOTATION_MARKERS: '/clear_annotation_markers',
        GET_ANNOTATION_MARKERS: '/get_annotation_markers',
        GET_ANNOTATION_MARKER_PLOT_OPTIONS: '/get_annotation_marker_plot_options',
        GET_COLORMAPS: '/get_colormaps',
        GET_DISTANCE_PLOT_OPTIONS: '/get_distance_plot_options',
        GET_FMRI_PLOT_OPTIONS: '/get_fmri_plot_options',
        GET_NIFTI_VIEW_STATE: '/get_nifti_view_state',
        GET_SELECTED_TIME_POINT: '/get_selected_time_point',
        GET_TASK_DESIGN_PLOT_OPTIONS: '/get_task_design_plot_options',
        GET_TIMECOURSE_PLOT_OPTIONS: '/get_timecourse_plot_options',
        GET_TIMECOURSE_GLOBAL_PLOT_OPTIONS: '/get_timecourse_global_plot_options',
        GET_TIMEMARKER_PLOT_OPTIONS: '/get_timemarker_plot_options',
        GET_TIMECOURSE_SHIFT_HISTORY: '/get_timecourse_shift_history',
        GET_TS_FMRI_PLOTTED: '/get_ts_fmri_plotted',
        MOVE_ANNOTATION_SELECTION: '/move_annotation_selection',
        REMOVE_DISTANCE_PLOT: '/remove_distance_plot',
        RESET_FMRI_COLOR_OPTIONS: '/reset_fmri_color_options',
        RESET_TIMECOURSE_SHIFT: '/reset_timecourse_shift',
        UPDATE_ANNOTATION_MARKER_PLOT_OPTIONS: '/update_annotation_marker_plot_options',
        UPDATE_DISTANCE_PLOT_OPTIONS: '/update_distance_plot_options',
        UPDATE_FMRI_PLOT_OPTIONS: '/update_fmri_plot_options',
        UPDATE_NIFTI_VIEW_STATE: '/update_nifti_view_state',
        UPDATE_TASK_DESIGN_PLOT_OPTIONS: '/update_task_design_plot_options',
        UPDATE_TIMECOURSE_PLOT_OPTIONS: '/update_timecourse_plot_options',
        UPDATE_TIMECOURSE_GLOBAL_PLOT_OPTIONS: '/update_timecourse_global_plot_options',
        UPDATE_TIMEMARKER_PLOT_OPTIONS: '/update_timemarker_plot_options',
        UPDATE_TIMECOURSE_SHIFT: '/update_timecourse_shift',
        UNDO_ANNOTATION_MARKER: '/undo_annotation_marker',
    },
    PREPROCESS: {
        GET_PREPROCESSED_FMRI: '/get_preprocessed_fmri',
        GET_PREPROCESSED_TIMECOURSE: '/get_preprocessed_timecourse',
        RESET_FMRI_PREPROCESS: '/reset_fmri_preprocess',
        RESET_TIMECOURSE_PREPROCESS: '/reset_timecourse_preprocess',
    },
    UPLOAD: {
        FILES: '/upload',
        HEADER: '/get_header',
        SCENE: '/upload_scene'
    }
};
